"""
Plots the simualtion results stored in this folder.
"""

import numpy as np
import pickle
import matplotlib as mpl
from matplotlib import pyplot as plt

plt.rcParams["figure.dpi"] = 200

pgf_with_latex = {  # setup matplotlib to use latex for output
    "pgf.texsystem": "pdflatex",  # change this if using xetex or lautex
    "text.usetex": True,  # use LaTeX to write all text
    "font.family": "serif",
    "font.serif": [],  # blank entries should cause plots
    "font.sans-serif": [],  # to inherit fonts from the document
    "font.monospace": [],
    "axes.labelsize": 30,
    "font.size": 50,
    "figure.figsize": [10, 10],
    "legend.fontsize": 20,  # Make the legend/label fonts
    "xtick.labelsize": 30,  # a little smaller
    "ytick.labelsize": 30,
    "pgf.preamble": [
        r"\usepackage[utf8]{inputenc}",  # use utf8 input and T1 fonts
        r"\usepackage[T1]{fontenc}",  # plots will be generated
    ]  # using this preamble
}
#mpl.use("pgf")
#mpl.rcParams.update(pgf_with_latex)


def save_object(_obj, _filename):
    with open(_filename, "wb") as output:
        pickle.dump(_obj, output, pickle.HIGHEST_PROTOCOL)


def load_object(_filename):
    with open(_filename, "rb") as input:
        return pickle.load(input)


def combine_maliciousness_arrays(_fa_array, _md_array, _type_key):
    try:
        mask = np.ones(len(_md_array[_type_key]["md"]), dtype=bool)
        mask[[0]] = False
        md_1_array = _md_array[_type_key]["md"][mask]
        combined = np.concatenate((md_1_array[::-1], _fa_array[_type_key]["fa"]))

        return combined

    except KeyError:
        print "Dict must have the valid keys."


if __name__ == "__main__":

    path = r"C:\Users\User\Dropbox\University\Masters\2019_Research\journal_article\results"

    # Choose which folder of results to plot.
    folder = ["trusted_users_sweep",
              "learning_speed",
              "mu_proportion_sweep_fa",
              "mu_proportion_sweep_md",
              "node_maliciousness_sweep_fa",
              "node_maliciousness_sweep_md",
              "mu_detection",
              "turning_off_factors",
              "crn_map",
              "iterations"]
    folder_choice = folder[3]
    print folder_choice

    if folder_choice == "node_maliciousness_sweep_fa":
        # False alarm simulation data.
        fa_1 = load_object(path + r"\node_maliciousness_sweep\maliciousness_sweep-0.3_fa_pop_50.pkl")
        fa_2 = load_object(path + r"\node_maliciousness_sweep\maliciousness_sweep-0.6_fa_pop_50.pkl")
        fa_3 = load_object(path + r"\node_maliciousness_sweep\maliciousness_sweep-0.9_fa_pop_30.pkl")
        print fa_3

        maliciousness_scale = np.linspace(0, 1, 11)
        plt.xlabel(r"Node maliciousness $(r)$")
        plt.ylabel(r"FA rate $(p_{fa})$")
        plt.plot(maliciousness_scale, fa_2["control"]["fa"], markersize=12, label=r"MDR (60% MUs)",
                 marker="D", color="lawngreen", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, fa_2["alg"]["fa"], markersize=12, label=r"PA (60% MUs)",
                 marker="D", color="fuchsia")
        plt.plot(maliciousness_scale, fa_3["control"], markersize=12, label=r"MDR (90% MUs)",
                 marker="o", color="lawngreen", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, fa_3["alg"], markersize=12, label=r"PA (90% MUs)",
                 marker="o", color="fuchsia")
        plt.legend()

        plt.grid()

        # plt.show()
        plt.savefig("maliciousness_sweep_60_percent_fa.pgf")

    elif folder_choice == "node_maliciousness_sweep_md":

        # Missed detection simulation data.
        md_2 = load_object(path + r"\node_maliciousness_sweep\maliciousness_sweep-0.6_md_pop_50.pkl")
        md_3 = load_object(path + r"\node_maliciousness_sweep\maliciousness_sweep-0.9_md_pop_30.pkl")

        maliciousness_scale = np.linspace(-1, 0, 11)
        plt.xlabel(r"Node maliciousness $(r)$")
        plt.ylabel(r"MD rate $(p_{md})$")
        plt.plot(maliciousness_scale, md_2["control"]["md"][::-1], markersize=12, label=r"MDR (60% MUs)",
                 marker="D", color="lawngreen", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, md_2["alg"]["md"][::-1], markersize=12, label=r"PA (60% MUs)",
                 marker="D", color="fuchsia")
        plt.plot(maliciousness_scale, md_3["control"], markersize=12, label=r"MDR (90% MUs)",
                 marker="o", color="lawngreen", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, md_3["alg"], markersize=12, label=r"PA (90% MUs)",
                 marker="o", color="fuchsia")
        plt.legend()
        plt.grid()

        # plt.show()
        plt.savefig("maliciousness_sweep_60_percent_md.pgf")

    elif folder_choice == "mu_proportion_sweep_fa":
        # Load the MU proportion sweep data.
        mu_prop_60 = load_object(path + r"\mu_proportion_sweep\propotion_sweep-mal_0.6_fa.pkl")
        mu_prop_90 = load_object(path + r"\mu_proportion_sweep\propotion_sweep-mal_0.9_fa.pkl")
        maliciousness_scale = np.linspace(0, 1, 11)
        plt.xlabel(r"Proportion of MUs")
        plt.ylabel(r"Error rate $(p_{fa})$")
        # plt.plot(maliciousness_scale, mu_prop_30["control"]["fa"][::-1], markersize=12, label=r"MDR ($r=0.3$)",
        #          marker="D", color="lawngreen")
        # plt.plot(maliciousness_scale, mu_prop_30["alg"]["fa"][::-1], markersize=12, label=r"PA ($r=0.3$)",
        #          marker="D", color="fuchsia")
        plt.plot(maliciousness_scale, mu_prop_60["control"]["fa"], markersize=12, label=r"MDR ($r=0.6$)",
                 marker="o", color="k", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, mu_prop_60["alg"]["fa"], markersize=12, label=r"PA ($r=0.6$)",
                 marker="o", color="k")
        plt.plot(maliciousness_scale, mu_prop_90["control"]["fa"], markersize=12, label=r"MDR ($r=0.9$)",
                 marker=10, color="k", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, mu_prop_90["alg"]["fa"], markersize=12, label=r"PA ($r=0.9$)",
                 marker=10, color="k")

        plt.legend()
        plt.grid()

        plt.savefig("mu_proportion_sweep_fa.eps")

    elif folder_choice == "mu_proportion_sweep_md":
        # Load the MU proportion sweep data.
        mu_prop_60 = load_object(path + r"\mu_proportion_sweep\mu_results_dictionary_0.6_50.pkl")
        mu_prop_90 = load_object(path + r"\mu_proportion_sweep\propotion_sweep-mal_0.9_md.pkl")
        maliciousness_scale = np.linspace(0, 1, 11)
        plt.xlabel(r"Proportion of MUs")
        plt.ylabel(r"Error rate $(p_{md})$")
        # plt.plot(maliciousness_scale, mu_prop_30["control"]["fa"][::-1], markersize=12, label=r"MDR ($r=0.3$)",
        #          marker="D", color="lawngreen")
        # plt.plot(maliciousness_scale, mu_prop_30["alg"]["fa"][::-1], markersize=12, label=r"PA ($r=0.3$)",
        #          marker="D", color="fuchsia")
        plt.plot(maliciousness_scale, mu_prop_60["control"]["md"], markersize=12, label=r"MDR ($r=-0.6$)",
                 marker="o", color="k", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, mu_prop_60["alg"]["md"], markersize=12, label=r"PA ($r=-0.6$)",
                 marker="o", color="k")
        plt.plot(maliciousness_scale, mu_prop_90["control"]["md"], markersize=12, label=r"MDR ($r=-0.9$)",
                 marker=10, color="k", linestyle="dashed", dashes=(5, 5))
        plt.plot(maliciousness_scale, mu_prop_90["alg"]["md"], markersize=12, label=r"PA ($r=-0.9$)",
                 marker=10, color="k")

        plt.legend()
        plt.grid()

        plt.savefig("mu_proportion_sweep_md.eps")

    elif folder_choice == "trusted_users_sweep":
        # Load the TU sweep data.
        tu_md = load_object(path + r"\trusted_users_sweep\mu_results_dictionary_trusted_15_mal_0.8_md.pkl")
        # tu_md = load_object(path + r"\trusted_users_sweep\trusted_sweep-mal_0.5_md_0.4_prop_10_users.pkl")
        # tu_fa_1 = load_object(path + r"\trusted_users_sweep\trusted_sweep-mal_0.8_fa_0.6_prop_10_users.pkl")
        # tu_md_1 = load_object(path + r"\trusted_users_sweep\trusted_sweep-mal_0.8_md_0.6_prop_10_users.pkl")
        maliciousness_scale = np.linspace(1, 15, 15)
        plt.xticks(np.arange(1, 16, step=2))

        plt.xlabel(r"Number of TUs")
        plt.ylabel(r"Error rate $(p_{e})$")

        plt.plot(maliciousness_scale, tu_md["alg"]["md"][:15] + tu_md["alg"]["fa"][:15], markersize=12, label=r"$r=-0.5$, 40% MUs",
                 marker=10, color="fuchsia")


        # plt.ylim([-0.06, 1])
        # plt.plot(maliciousness_scale, tu_fa["control"]["fa"], markersize=12, label=r"MDR ($r=0.8$)",
        #          marker="o", color="lawngreen")
        # plt.plot(maliciousness_scale, tu_fa["alg"]["fa"] + tu_fa["alg"]["md"], markersize=12, label=r"$r=0.5$, 40% MUs",
        #          marker="o", color="fuchsia")

        # plt.plot(maliciousness_scale, tu_fa_1["alg"]["fa"] + tu_fa_1["alg"]["md"], markersize=12, label=r"$r=0.8$, 60% MUs",
        #          marker="o", color="lawngreen", linestyle="dashed", dashes=(5, 5))
        # plt.plot(maliciousness_scale, tu_md_1["alg"]["md"] + tu_md_1["alg"]["fa"], markersize=12, label=r"$r=-0.8$, 60% MUs",
        #          marker=10, color="lawngreen", linestyle="dashed", dashes=(5, 5))

        plt.legend()
        plt.grid()

        plt.savefig("trusted_users_sweep.pgf")

    elif folder_choice == "learning_speed":
        # Load the learning speed data.
        learning_fa = load_object(path + r"\learning_speed\learning_speed_fa_75.pkl")
        learning_md = load_object(path + r"\learning_speed\learning_speed_md_75.pkl")
        plt.xlabel(r"Time instants")
        plt.ylabel(r"MD rate $(p_{md})$")
        learning_time_scale = np.linspace(1, 30, 15)
        # plt.ylim([0, 1])
        plt.xticks(np.arange(2, 34, step=4))
        plt.plot(learning_time_scale, learning_md["control"][::2], markersize=12, label=r"MDR",
                 marker="o", color="lawngreen")
        plt.plot(learning_time_scale, learning_md["alg"][::2], markersize=12, label=r"PA",
                 marker=10, color="fuchsia")
        plt.legend()
        plt.grid()

        plt.savefig("learning_speed_md.pgf")

    # elif folder_choice == "mu_detection":
    #     # Load the MU detection data.
    #     mu_detection_noise_15

    elif folder_choice == "turning_off_factors":
        # Load the turning off factors data.
        turning_off_gamma = load_object(path + r"\turning_off_factors\turning_off_gamma_prop_0.9_md.pkl")
        turning_off_delta_a = load_object(path + r"\turning_off_factors\turning_off_delta_a_prop_0.9_md.pkl")
        turning_off_delta_b = load_object(path + r"\turning_off_factors\turning_off_delta_b_prop_0.9_md.pkl")
        turning_off_alpha_k = load_object(path + r"\turning_off_factors\turning_off_alpha_k_prop_0.9_md.pkl")
        turning_off_alpha_m = load_object(path + r"\turning_off_factors\turning_off_alpha_m_prop_0.9_md.pkl")

        turning_off_sum = turning_off_gamma["alg"]["md"] \
                          + turning_off_delta_a["alg"]["md"] \
                          + turning_off_delta_b["alg"]["md"] \
                          + turning_off_alpha_k["alg"]["md"] \
                          + turning_off_alpha_m["alg"]["md"]


        proportion_scale = np.linspace(0, 1, 11)
        plt.xlabel(r"Proportion of MUs ($\%$)")
        plt.ylabel(r"MD rate $(p_{md})$")


        # plt.plot(maliciousness_scale, mu_prop_30["control"]["fa"][::-1], markersize=12, label=r"MDR ($r=0.3$)",
        #          marker="D", color="lawngreen")
        # plt.plot(maliciousness_scale, mu_prop_30["alg"]["fa"][::-1], markersize=12, label=r"PA ($r=0.3$)",
        #          marker="D", color="fuchsia")

        # plt.ylim([-0.1, 1])

        plt.plot(proportion_scale, turning_off_gamma["alg"]["md"], markersize=12, label=r"$~\gamma_{mk}$",
                 marker=10, color="crimson")
        plt.plot(proportion_scale, turning_off_delta_a["alg"]["md"], markersize=12, label=r"$~\delta_a$",
                 marker=10, color="lawngreen")
        plt.plot(proportion_scale, turning_off_delta_b["alg"]["md"], markersize=12, label=r"$~\delta_b$",
                 marker=10, color="midnightblue")
        plt.plot(proportion_scale, turning_off_alpha_k["alg"]["md"], markersize=12, label=r"$~\alpha_k$",
                 marker=10, color="dimgrey")
        # plt.plot(proportion_scale, turning_off_sum, markersize=12, label=r"$~\sum$",
        #          marker=10, color="springgreen")
        plt.plot(proportion_scale, turning_off_alpha_m["alg"]["md"], markersize=12, label=r"$~\alpha_m$",
                 marker=10, color="r")

        plt.legend()
        plt.grid()

        plt.savefig("turning_off_factors.pgf")

    elif folder_choice == "mu_detection":
        # Load the turning off factors data.
        mu_detection_results = load_object(path + r"\mu_detection\type4fg_results_50_noise-20-15_users.pkl")
        mu_beliefs = mu_detection_results["beliefs"]
        malicious_nodes = mu_detection_results["m_nodes"]
        node_maliciousness = mu_detection_results["maliciousness"]
        maliciousness_scale = np.linspace(-1, 1, 202)

        print malicious_nodes
        print node_maliciousness
        print mu_beliefs

        color_cycle = ["crimson", "lawngreen", "midnightblue", "dimgrey", "springgreen", "k"]
        line_cycle = ["solid", "dotted", "dashed", "dashdot"]

        for i, nodes in enumerate(malicious_nodes):
            temp_str = "r_" + str(nodes)
            plt.plot(maliciousness_scale, mu_beliefs[temp_str], markersize=12, linewidth=4.0,
                     label="$" + temp_str + "$", color="k", linestyle=line_cycle[i])
            plt.scatter(np.array([node_maliciousness[i]]), np.array([np.max(mu_beliefs[temp_str])]), marker="o",
                        s=100, color="k")
            plt.vlines(x=node_maliciousness[i], ymin=0, ymax=np.max(mu_beliefs[temp_str]),
                       color="k", linewidth=4.0, linestyle=line_cycle[i])
            # plt.ylim([-0.1, 10])
            plt.xlabel(r"Node maliciousness ($r$)")
            plt.ylabel(r"Probability density")

        plt.legend()
        plt.grid()
        plt.savefig("mu_detection_20.eps")

    elif folder_choice == "crn_map":
        crn_map_dict = load_object(path + r"\trusted_users_sweep\tu_sim_crn_map_data.pkl")
        trusted_su = crn_map_dict["trusted_su"]
        num_su = crn_map_dict["num_su"]
        malicious_nodes = crn_map_dict["malicious_nodes"]
        su_pos = crn_map_dict["su_pos"]

        fig = plt.figure()
        axes = fig.add_subplot(1, 1, 1)

        # Plot the crn map.
        axes.scatter(su_pos[0, np.arange(num_su)],
                     su_pos[1, np.arange(num_su)], c="green", marker="o", s=400, label="Honest UTUs")

        axes.scatter(su_pos[0, malicious_nodes],
                     su_pos[1, malicious_nodes], c="red", marker="o", s=400, label="MUs")

        axes.scatter(su_pos[0][trusted_su],
                     su_pos[1][trusted_su], c="cyan",
                     marker="o",
                     s=600, label="TUs")

        axes.scatter(32.5,
                     32.5, c="k", marker=r"X", s=200, label="PU Location")

        for i, tu_idx in enumerate(trusted_su):
            axes.scatter(su_pos[0][tu_idx],
                         su_pos[1][tu_idx], c="k",
                         marker=r"$ {} $".format(i + 1),
                         s=500)

        axes.set_xlim(0, 75)
        axes.set_ylim(0, 75)

        axes.set_xlabel(r"$x$")
        axes.set_ylabel(r"$y$")

        axes.grid()
        plt.legend()
        plt.savefig("crn_map_trusted.pgf")
        plt.show()

    elif folder_choice == "iterations":
        # iteration_sim = load_object(path + r"\iterations\iteration_sim_0.5_50_percent.pkl")
        iteration_progress_0 = load_object(path + r"\iterations\iteration_progress_0.pkl")
        iteration_progress_1 = load_object(path + r"\iterations\iteration_progress_1.pkl")

        color_cycle = ["crimson", "lawngreen", "midnightblue", "dimgrey", "springgreen", "k"]
        line_cycle = ["solid", "dotted", "dashed", "dashdot"]

        # Plot iteration progress.
        iteration_range = np.linspace(1, 7, 7, dtype=np.int)
        belief_range_0 = iteration_progress_0["beliefs"]
        pu_state_0 = iteration_progress_0["pu_state"]

        belief_range_1 = iteration_progress_1["beliefs"]
        pu_state_1 = iteration_progress_1["pu_state"]

        results_dict = {"beliefs": belief_range_0, "pu_state": pu_state_0}
        plt.ylim([-0.1, 1.1])
        plt.xlabel(r"Iterations")
        plt.ylabel(r"Probability density")

        # Plot when PU is 0.
        plt.hlines(y=pu_state_0, xmin=0, xmax=np.shape(iteration_range)[0], color=color_cycle[0],
                   linewidth=4.0, linestyle=line_cycle[0], label=r"$\mathcal{H}^{(t)} = 0$")
        plt.scatter(iteration_range, belief_range_0, color=color_cycle[0], linestyle=line_cycle[0],
                    label=r"PA $b(\mathcal{H}^{(t)})=0$", s=200)

        # Plot when PU is 1.
        plt.hlines(y=pu_state_1, xmin=0, xmax=np.shape(iteration_range)[0], color=color_cycle[1],
                   linewidth=4.0, linestyle=line_cycle[1], label=r"$\mathcal{H}^{(t)} = 1$")
        plt.scatter(iteration_range, belief_range_1, color=color_cycle[1], linestyle=line_cycle[1],
                    label=r"PA $b(\mathcal{H}^{(t)})=1$", s=200)
        plt.xticks(np.arange(1, 8, step=1))
        plt.grid()
        plt.legend()
        plt.savefig("iteration_progress.pgf")
        # plt.show()

    else:
        pass
